import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;

public class Date {

	private static final int daysInMonths[]={31,28,31,30,31,30,31,31,30,31,30,31};
	private static final String monthInFrench[]={ 
		"Janvier", "Fvrier", "Mars",
		"Avril", "Mai", "Juin", "Juillet", "Aout",
		"Septembre", "Octobre", "Novembre", "Dcembre"
	};
	private static final String dayInFrench[]={
	"Lundi", "Mardi", "Mercredi",
	"Jeudi", "Vendredi", "Samedi",
	"Dimanche"
	};
	
	private int day = 1;
	private int month = 1;
	private int year = 1970;	
	
	/**
	 * Constructs a new Date object representing the date day-month-year. 
	 */	
	public Date(int day, int month, int year)
	{
		setYear(year);
		setMonth(month);
		setDay(day);
	}
	
	public int getDay()
	{
		return this.day;
	}
	
	public int getMonth()
	{
		return this.month;
	}
	
	public int getYear()
	{
		return this.year;
	}
	
	public void setYear(int year)
	{
		if(isLeapYear(this.year) && !isLeapYear(year) && lastDayOfMonth()){
			throw new RuntimeException();
		}
		else{
			this.year=year;
		}
	}
	
	/**
	 * Sets the month. 
	 */
	public void setMonth(int month)
	{
		if(month>=1 && month <=12) 
			this.month = month;
		else{
			throw new RuntimeException("bad month");
		}
			
	}
	
	/**
	 * Sets the day. 
	 * Year and Month must be set already BEFORE calling this method.
	 */
	public void setDay(int day)
	{
		if(day>=1 && day<=this.daysInMonth()) 
			this.day=day;
		else{
			throw new RuntimeException("bad day");
		}
	}
	
	/**
	 *  Increments the date of one day
	 **/
	public void increment()
	{
		if(lastDayOfMonth()) {
			day = 1;
			if(month == 12){
				month = 1; 
				year++;
			} else {
				month++;
			}
		} else {
			day++;
		}
	}
	
	/**
	 *  @returns The number of days in the month this date belongs to.
	 **/
	public static int daysInMonth(int month, int year)
	{
		return daysInMonths[month-1] 
				+ (isLeapYear(year) && month==2? 1 : 0 );
	}
	
	/**
	 *  @returns The number of days in the month this date belongs to.
	 **/
	private int daysInMonth()
	{
		return daysInMonth(this.month, this.year);
	}
	
	/**
	 * @returns true if this is the last day of the month.
	 */
	private boolean lastDayOfMonth()
	{
		return day == daysInMonth();
	}
	
	public static boolean isLeapYear(int year)
	{
		return year%400==0 || (year%100 !=0 && year%4==0);
	}
	
	/**
	 * @returns true if it is a leap year.
	 */
	private boolean isLeapYear()
	{
		return isLeapYear(this.year);
	}
	
	/**
	 * @returns the day number in the current year this date represents.
	 */	
	public int dayOfYear()
	{
		int dayOfYear=this.day;
		for(int i=0; i<month-1;i++)
			dayOfYear += daysInMonths[i];
		if(isLeapYear() && month>2) dayOfYear++;
		return dayOfYear;
	}
	
	/**
	* @returns 0=monday, 1=tuesday, ..., 6=sunday
	* see http://en.wikipedia.org/wiki/Zeller's_congruence
	*/	
	public int dayOfWeek()
	{
		int m = this.month; // local copies because
		int y = this.year;  // month and year can be modified
		if(m==1 || m==2)
		{
			m += 12;
			y--;            
		}
		
		int century = y/100;
		int yearOfCentury = y%100;
		int dayOfWeek = (day + 
						 + (((m+1)*26)/10)
						 + yearOfCentury 
						 + (yearOfCentury/4) 
						 + (century/4) 
						 + 5*century) % 7;
		
		return (dayOfWeek+5)%7;
	}
	
	
	public String toString()
	{
		return dayInFrench[dayOfWeek()]+" "+day+" "+ 
		       monthInFrench[month-1]+" "+year+ 
			   " le "+dayOfYear()+"-ime jour de l'anne";
	}
	
	public void writeToFile(String fileName){
		try {
			BufferedWriter out = new BufferedWriter(new FileWriter(fileName));
			out.write(this.toString());
			out.close();
		} catch (IOException e) {
			System.out.println("Problme lors de l'enregisrement de la date : \n"+e.getMessage());
		}
	}
	
	public static void main(String args[])
	{
		Date today = new Date(2, 02, 2012);
		System.out.println(today);
		Date d2 = new Date(29,02,2012);
		d2.increment();  //aprs le 29/02 -> 01/03
		System.out.println(d2);
		d2.writeToFile("date.txt");
		System.out.println(d2);
		Date d3 = new Date(30,02,2012); //date inexistante -> exception
		System.out.println(d3);

	}
}
